"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCryptoStoreCacheCallbacks = createCryptoStoreCacheCallbacks;
exports.DeviceTrustLevel = exports.UserTrustLevel = exports.CrossSigningLevel = exports.CrossSigningInfo = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _olmlib = require("./olmlib");

var _events = require("events");

var _logger = require("../logger");

var _indexeddbCryptoStore = require("../crypto/store/indexeddb-crypto-store");

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = (0, _getPrototypeOf2["default"])(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = (0, _getPrototypeOf2["default"])(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return (0, _possibleConstructorReturn2["default"])(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function publicKeyFromKeyInfo(keyInfo) {
  // `keys` is an object with { [`ed25519:${pubKey}`]: pubKey }
  // We assume only a single key, and we want the bare form without type
  // prefix, so we select the values.
  return Object.values(keyInfo.keys)[0];
}

var CrossSigningInfo = /*#__PURE__*/function (_EventEmitter) {
  (0, _inherits2["default"])(CrossSigningInfo, _EventEmitter);

  var _super = _createSuper(CrossSigningInfo);

  /**
   * Information about a user's cross-signing keys
   *
   * @class
   *
   * @param {string} userId the user that the information is about
   * @param {object} callbacks Callbacks used to interact with the app
   *     Requires getCrossSigningKey and saveCrossSigningKeys
   * @param {object} cacheCallbacks Callbacks used to interact with the cache
   */
  function CrossSigningInfo(userId, callbacks, cacheCallbacks) {
    var _this;

    (0, _classCallCheck2["default"])(this, CrossSigningInfo);
    _this = _super.call(this); // you can't change the userId

    Object.defineProperty((0, _assertThisInitialized2["default"])(_this), 'userId', {
      enumerable: true,
      value: userId
    });
    _this._callbacks = callbacks || {};
    _this._cacheCallbacks = cacheCallbacks || {};
    _this.keys = {};
    _this.firstUse = true; // This tracks whether we've ever verified this user with any identity.
    // When you verify a user, any devices online at the time that receive
    // the verifying signature via the homeserver will latch this to true
    // and can use it in the future to detect cases where the user has
    // become unverifed later for any reason.

    _this.crossSigningVerifiedBefore = false;
    return _this;
  }
  /**
   * Calls the app callback to ask for a private key
   * @param {string} type The key type ("master", "self_signing", or "user_signing")
   * @param {string} expectedPubkey The matching public key or undefined to use
   *     the stored public key for the given key type.
   * @returns {Array} An array with [ public key, Olm.PkSigning ]
   */


  (0, _createClass2["default"])(CrossSigningInfo, [{
    key: "getCrossSigningKey",
    value: function () {
      var _getCrossSigningKey = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(type, expectedPubkey) {
        var shouldCache, validateKey, privkey, cacheresult, result;
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                validateKey = function _validateKey(key) {
                  if (!key) return;
                  var signing = new global.Olm.PkSigning();
                  var gotPubkey = signing.init_with_seed(key);

                  if (gotPubkey === expectedPubkey) {
                    return [gotPubkey, signing];
                  }

                  signing.free();
                };

                shouldCache = ["self_signing", "user_signing"].indexOf(type) >= 0;

                if (this._callbacks.getCrossSigningKey) {
                  _context.next = 4;
                  break;
                }

                throw new Error("No getCrossSigningKey callback supplied");

              case 4:
                if (expectedPubkey === undefined) {
                  expectedPubkey = this.getId(type);
                }

                if (!(this._cacheCallbacks.getCrossSigningKeyCache && shouldCache)) {
                  _context.next = 9;
                  break;
                }

                _context.next = 8;
                return this._cacheCallbacks.getCrossSigningKeyCache(type, expectedPubkey);

              case 8:
                privkey = _context.sent;

              case 9:
                cacheresult = validateKey(privkey);

                if (!cacheresult) {
                  _context.next = 12;
                  break;
                }

                return _context.abrupt("return", cacheresult);

              case 12:
                _context.next = 14;
                return this._callbacks.getCrossSigningKey(type, expectedPubkey);

              case 14:
                privkey = _context.sent;
                result = validateKey(privkey);

                if (!result) {
                  _context.next = 21;
                  break;
                }

                if (!(this._cacheCallbacks.storeCrossSigningKeyCache && shouldCache)) {
                  _context.next = 20;
                  break;
                }

                _context.next = 20;
                return this._cacheCallbacks.storeCrossSigningKeyCache(type, privkey);

              case 20:
                return _context.abrupt("return", result);

              case 21:
                if (privkey) {
                  _context.next = 23;
                  break;
                }

                throw new Error("getCrossSigningKey callback for " + type + " returned falsey");

              case 23:
                throw new Error("Key type " + type + " from getCrossSigningKey callback did not match");

              case 24:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getCrossSigningKey(_x, _x2) {
        return _getCrossSigningKey.apply(this, arguments);
      }

      return getCrossSigningKey;
    }()
  }, {
    key: "toStorage",
    value: function toStorage() {
      return {
        keys: this.keys,
        firstUse: this.firstUse,
        crossSigningVerifiedBefore: this.crossSigningVerifiedBefore
      };
    }
    /**
     * Check whether the private keys exist in secret storage.
     * XXX: This could be static, be we often seem to have an instance when we
     * want to know this anyway...
     *
     * @param {SecretStorage} secretStorage The secret store using account data
     * @returns {object} map of key name to key info the secret is encrypted
     *     with, or null if it is not present or not encrypted with a trusted
     *     key
     */

  }, {
    key: "isStoredInSecretStorage",
    value: function () {
      var _isStoredInSecretStorage = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(secretStorage) {
        var stored, intersect, _i2, _arr, type;

        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                intersect = function _intersect(s) {
                  for (var _i = 0, _Object$keys = Object.keys(stored); _i < _Object$keys.length; _i++) {
                    var k = _Object$keys[_i];

                    if (!s[k]) {
                      delete stored[k];
                    }
                  }
                };

                _context2.next = 3;
                return secretStorage.isStored("m.cross_signing.master", false);

              case 3:
                _context2.t0 = _context2.sent;

                if (_context2.t0) {
                  _context2.next = 6;
                  break;
                }

                _context2.t0 = {};

              case 6:
                stored = _context2.t0;
                _i2 = 0, _arr = ["self_signing", "user_signing"];

              case 8:
                if (!(_i2 < _arr.length)) {
                  _context2.next = 21;
                  break;
                }

                type = _arr[_i2];
                _context2.t1 = intersect;
                _context2.next = 13;
                return secretStorage.isStored("m.cross_signing.".concat(type), false);

              case 13:
                _context2.t2 = _context2.sent;

                if (_context2.t2) {
                  _context2.next = 16;
                  break;
                }

                _context2.t2 = {};

              case 16:
                _context2.t3 = _context2.t2;
                (0, _context2.t1)(_context2.t3);

              case 18:
                _i2++;
                _context2.next = 8;
                break;

              case 21:
                return _context2.abrupt("return", Object.keys(stored).length ? stored : null);

              case 22:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }));

      function isStoredInSecretStorage(_x3) {
        return _isStoredInSecretStorage.apply(this, arguments);
      }

      return isStoredInSecretStorage;
    }()
    /**
     * Store private keys in secret storage for use by other devices. This is
     * typically called in conjunction with the creation of new cross-signing
     * keys.
     *
     * @param {object} keys The keys to store
     * @param {SecretStorage} secretStorage The secret store using account data
     */

  }, {
    key: "getId",

    /**
     * Get the ID used to identify the user. This can also be used to test for
     * the existence of a given key type.
     *
     * @param {string} type The type of key to get the ID of.  One of "master",
     * "self_signing", or "user_signing".  Defaults to "master".
     *
     * @return {string} the ID
     */
    value: function getId(type) {
      type = type || "master";
      if (!this.keys[type]) return null;
      var keyInfo = this.keys[type];
      return publicKeyFromKeyInfo(keyInfo);
    }
    /**
     * Create new cross-signing keys for the given key types. The public keys
     * will be held in this class, while the private keys are passed off to the
     * `saveCrossSigningKeys` application callback.
     *
     * @param {CrossSigningLevel} level The key types to reset
     */

  }, {
    key: "resetKeys",
    value: function () {
      var _resetKeys = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(level) {
        var privateKeys, keys, masterSigning, masterPub, _yield$this$getCrossS, _yield$this$getCrossS2, sskSigning, sskPub, uskSigning, uskPub;

        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (this._callbacks.saveCrossSigningKeys) {
                  _context3.next = 2;
                  break;
                }

                throw new Error("No saveCrossSigningKeys callback supplied");

              case 2:
                if (!(level === undefined || level & CrossSigningLevel.MASTER || !this.keys.master)) {
                  _context3.next = 6;
                  break;
                }

                level = CrossSigningLevel.MASTER | CrossSigningLevel.USER_SIGNING | CrossSigningLevel.SELF_SIGNING;
                _context3.next = 8;
                break;

              case 6:
                if (!(level === 0)) {
                  _context3.next = 8;
                  break;
                }

                return _context3.abrupt("return");

              case 8:
                privateKeys = {};
                keys = {};
                _context3.prev = 10;

                if (!(level & CrossSigningLevel.MASTER)) {
                  _context3.next = 18;
                  break;
                }

                masterSigning = new global.Olm.PkSigning();
                privateKeys.master = masterSigning.generate_seed();
                masterPub = masterSigning.init_with_seed(privateKeys.master);
                keys.master = {
                  user_id: this.userId,
                  usage: ['master'],
                  keys: (0, _defineProperty2["default"])({}, 'ed25519:' + masterPub, masterPub)
                };
                _context3.next = 24;
                break;

              case 18:
                _context3.next = 20;
                return this.getCrossSigningKey("master");

              case 20:
                _yield$this$getCrossS = _context3.sent;
                _yield$this$getCrossS2 = (0, _slicedToArray2["default"])(_yield$this$getCrossS, 2);
                masterPub = _yield$this$getCrossS2[0];
                masterSigning = _yield$this$getCrossS2[1];

              case 24:
                if (level & CrossSigningLevel.SELF_SIGNING) {
                  sskSigning = new global.Olm.PkSigning();

                  try {
                    privateKeys.self_signing = sskSigning.generate_seed();
                    sskPub = sskSigning.init_with_seed(privateKeys.self_signing);
                    keys.self_signing = {
                      user_id: this.userId,
                      usage: ['self_signing'],
                      keys: (0, _defineProperty2["default"])({}, 'ed25519:' + sskPub, sskPub)
                    };
                    (0, _olmlib.pkSign)(keys.self_signing, masterSigning, this.userId, masterPub);
                  } finally {
                    sskSigning.free();
                  }
                }

                if (level & CrossSigningLevel.USER_SIGNING) {
                  uskSigning = new global.Olm.PkSigning();

                  try {
                    privateKeys.user_signing = uskSigning.generate_seed();
                    uskPub = uskSigning.init_with_seed(privateKeys.user_signing);
                    keys.user_signing = {
                      user_id: this.userId,
                      usage: ['user_signing'],
                      keys: (0, _defineProperty2["default"])({}, 'ed25519:' + uskPub, uskPub)
                    };
                    (0, _olmlib.pkSign)(keys.user_signing, masterSigning, this.userId, masterPub);
                  } finally {
                    uskSigning.free();
                  }
                }

                Object.assign(this.keys, keys);

                this._callbacks.saveCrossSigningKeys(privateKeys);

              case 28:
                _context3.prev = 28;

                if (masterSigning) {
                  masterSigning.free();
                }

                return _context3.finish(28);

              case 31:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this, [[10,, 28, 31]]);
      }));

      function resetKeys(_x4) {
        return _resetKeys.apply(this, arguments);
      }

      return resetKeys;
    }()
    /**
     * unsets the keys, used when another session has reset the keys, to disable cross-signing
     */

  }, {
    key: "clearKeys",
    value: function clearKeys() {
      this.keys = {};
    }
  }, {
    key: "setKeys",
    value: function setKeys(keys) {
      var signingKeys = {};

      if (keys.master) {
        if (keys.master.user_id !== this.userId) {
          var error = "Mismatched user ID " + keys.master.user_id + " in master key from " + this.userId;

          _logger.logger.error(error);

          throw new Error(error);
        }

        if (!this.keys.master) {
          // this is the first key we've seen, so first-use is true
          this.firstUse = true;
        } else if (publicKeyFromKeyInfo(keys.master) !== this.getId()) {
          // this is a different key, so first-use is false
          this.firstUse = false;
        } // otherwise, same key, so no change


        signingKeys.master = keys.master;
      } else if (this.keys.master) {
        signingKeys.master = this.keys.master;
      } else {
        throw new Error("Tried to set cross-signing keys without a master key");
      }

      var masterKey = publicKeyFromKeyInfo(signingKeys.master); // verify signatures

      if (keys.user_signing) {
        if (keys.user_signing.user_id !== this.userId) {
          var _error = "Mismatched user ID " + keys.master.user_id + " in user_signing key from " + this.userId;

          _logger.logger.error(_error);

          throw new Error(_error);
        }

        try {
          (0, _olmlib.pkVerify)(keys.user_signing, masterKey, this.userId);
        } catch (e) {
          _logger.logger.error("invalid signature on user-signing key"); // FIXME: what do we want to do here?


          throw e;
        }
      }

      if (keys.self_signing) {
        if (keys.self_signing.user_id !== this.userId) {
          var _error2 = "Mismatched user ID " + keys.master.user_id + " in self_signing key from " + this.userId;

          _logger.logger.error(_error2);

          throw new Error(_error2);
        }

        try {
          (0, _olmlib.pkVerify)(keys.self_signing, masterKey, this.userId);
        } catch (e) {
          _logger.logger.error("invalid signature on self-signing key"); // FIXME: what do we want to do here?


          throw e;
        }
      } // if everything checks out, then save the keys


      if (keys.master) {
        this.keys.master = keys.master; // if the master key is set, then the old self-signing and
        // user-signing keys are obsolete

        this.keys.self_signing = null;
        this.keys.user_signing = null;
      }

      if (keys.self_signing) {
        this.keys.self_signing = keys.self_signing;
      }

      if (keys.user_signing) {
        this.keys.user_signing = keys.user_signing;
      }
    }
  }, {
    key: "updateCrossSigningVerifiedBefore",
    value: function updateCrossSigningVerifiedBefore(isCrossSigningVerified) {
      // It is critical that this value latches forward from false to true but
      // never back to false to avoid a downgrade attack.
      if (!this.crossSigningVerifiedBefore && isCrossSigningVerified) {
        this.crossSigningVerifiedBefore = true;
      }
    }
  }, {
    key: "signObject",
    value: function () {
      var _signObject = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(data, type) {
        var _yield$this$getCrossS3, _yield$this$getCrossS4, pubkey, signing;

        return _regenerator["default"].wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                if (this.keys[type]) {
                  _context4.next = 2;
                  break;
                }

                throw new Error("Attempted to sign with " + type + " key but no such key present");

              case 2:
                _context4.next = 4;
                return this.getCrossSigningKey(type);

              case 4:
                _yield$this$getCrossS3 = _context4.sent;
                _yield$this$getCrossS4 = (0, _slicedToArray2["default"])(_yield$this$getCrossS3, 2);
                pubkey = _yield$this$getCrossS4[0];
                signing = _yield$this$getCrossS4[1];
                _context4.prev = 8;
                (0, _olmlib.pkSign)(data, signing, this.userId, pubkey);
                return _context4.abrupt("return", data);

              case 11:
                _context4.prev = 11;
                signing.free();
                return _context4.finish(11);

              case 14:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this, [[8,, 11, 14]]);
      }));

      function signObject(_x5, _x6) {
        return _signObject.apply(this, arguments);
      }

      return signObject;
    }()
  }, {
    key: "signUser",
    value: function () {
      var _signUser = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(key) {
        return _regenerator["default"].wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                if (this.keys.user_signing) {
                  _context5.next = 3;
                  break;
                }

                _logger.logger.info("No user signing key: not signing user");

                return _context5.abrupt("return");

              case 3:
                return _context5.abrupt("return", this.signObject(key.keys.master, "user_signing"));

              case 4:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function signUser(_x7) {
        return _signUser.apply(this, arguments);
      }

      return signUser;
    }()
  }, {
    key: "signDevice",
    value: function () {
      var _signDevice = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(userId, device) {
        return _regenerator["default"].wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                if (!(userId !== this.userId)) {
                  _context6.next = 2;
                  break;
                }

                throw new Error("Trying to sign ".concat(userId, "'s device; can only sign our own device"));

              case 2:
                if (this.keys.self_signing) {
                  _context6.next = 5;
                  break;
                }

                _logger.logger.info("No self signing key: not signing device");

                return _context6.abrupt("return");

              case 5:
                return _context6.abrupt("return", this.signObject({
                  algorithms: device.algorithms,
                  keys: device.keys,
                  device_id: device.deviceId,
                  user_id: userId
                }, "self_signing"));

              case 6:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function signDevice(_x8, _x9) {
        return _signDevice.apply(this, arguments);
      }

      return signDevice;
    }()
    /**
     * Check whether a given user is trusted.
     *
     * @param {CrossSigningInfo} userCrossSigning Cross signing info for user
     *
     * @returns {UserTrustLevel}
     */

  }, {
    key: "checkUserTrust",
    value: function checkUserTrust(userCrossSigning) {
      // if we're checking our own key, then it's trusted if the master key
      // and self-signing key match
      if (this.userId === userCrossSigning.userId && this.getId() && this.getId() === userCrossSigning.getId() && this.getId("self_signing") && this.getId("self_signing") === userCrossSigning.getId("self_signing")) {
        return new UserTrustLevel(true, true, this.firstUse);
      }

      if (!this.keys.user_signing) {
        // If there's no user signing key, they can't possibly be verified.
        // They may be TOFU trusted though.
        return new UserTrustLevel(false, false, userCrossSigning.firstUse);
      }

      var userTrusted;
      var userMaster = userCrossSigning.keys.master;
      var uskId = this.getId('user_signing');

      try {
        (0, _olmlib.pkVerify)(userMaster, uskId, this.userId);
        userTrusted = true;
      } catch (e) {
        userTrusted = false;
      }

      return new UserTrustLevel(userTrusted, userCrossSigning.crossSigningVerifiedBefore, userCrossSigning.firstUse);
    }
    /**
     * Check whether a given device is trusted.
     *
     * @param {CrossSigningInfo} userCrossSigning Cross signing info for user
     * @param {module:crypto/deviceinfo} device The device to check
     * @param {bool} localTrust Whether the device is trusted locally
     * @param {bool} trustCrossSignedDevices Whether we trust cross signed devices
     *
     * @returns {DeviceTrustLevel}
     */

  }, {
    key: "checkDeviceTrust",
    value: function checkDeviceTrust(userCrossSigning, device, localTrust, trustCrossSignedDevices) {
      var userTrust = this.checkUserTrust(userCrossSigning);
      var userSSK = userCrossSigning.keys.self_signing;

      if (!userSSK) {
        // if the user has no self-signing key then we cannot make any
        // trust assertions about this device from cross-signing
        return new DeviceTrustLevel(false, false, localTrust, trustCrossSignedDevices);
      }

      var deviceObj = deviceToObject(device, userCrossSigning.userId);

      try {
        // if we can verify the user's SSK from their master key...
        (0, _olmlib.pkVerify)(userSSK, userCrossSigning.getId(), userCrossSigning.userId); // ...and this device's key from their SSK...

        (0, _olmlib.pkVerify)(deviceObj, publicKeyFromKeyInfo(userSSK), userCrossSigning.userId); // ...then we trust this device as much as far as we trust the user

        return DeviceTrustLevel.fromUserTrustLevel(userTrust, localTrust, trustCrossSignedDevices);
      } catch (e) {
        return new DeviceTrustLevel(false, false, localTrust, trustCrossSignedDevices);
      }
    }
    /**
     * @returns {object} Cache callbacks
     */

  }, {
    key: "getCacheCallbacks",
    value: function getCacheCallbacks() {
      return this._cacheCallbacks;
    }
  }], [{
    key: "fromStorage",
    value: function fromStorage(obj, userId) {
      var res = new CrossSigningInfo(userId);

      for (var prop in obj) {
        if (obj.hasOwnProperty(prop)) {
          res[prop] = obj[prop];
        }
      }

      return res;
    }
  }, {
    key: "storeInSecretStorage",
    value: function () {
      var _storeInSecretStorage = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(keys, secretStorage) {
        var _i3, _Object$keys2, type, encodedKey;

        return _regenerator["default"].wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _i3 = 0, _Object$keys2 = Object.keys(keys);

              case 1:
                if (!(_i3 < _Object$keys2.length)) {
                  _context7.next = 9;
                  break;
                }

                type = _Object$keys2[_i3];
                encodedKey = (0, _olmlib.encodeBase64)(keys[type]);
                _context7.next = 6;
                return secretStorage.store("m.cross_signing.".concat(type), encodedKey);

              case 6:
                _i3++;
                _context7.next = 1;
                break;

              case 9:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7);
      }));

      function storeInSecretStorage(_x10, _x11) {
        return _storeInSecretStorage.apply(this, arguments);
      }

      return storeInSecretStorage;
    }()
    /**
     * Get private keys from secret storage created by some other device. This
     * also passes the private keys to the app-specific callback.
     *
     * @param {string} type The type of key to get.  One of "master",
     * "self_signing", or "user_signing".
     * @param {SecretStorage} secretStorage The secret store using account data
     * @return {Uint8Array} The private key
     */

  }, {
    key: "getFromSecretStorage",
    value: function () {
      var _getFromSecretStorage = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(type, secretStorage) {
        var encodedKey;
        return _regenerator["default"].wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _context8.next = 2;
                return secretStorage.get("m.cross_signing.".concat(type));

              case 2:
                encodedKey = _context8.sent;
                return _context8.abrupt("return", (0, _olmlib.decodeBase64)(encodedKey));

              case 4:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8);
      }));

      function getFromSecretStorage(_x12, _x13) {
        return _getFromSecretStorage.apply(this, arguments);
      }

      return getFromSecretStorage;
    }()
  }]);
  return CrossSigningInfo;
}(_events.EventEmitter);

exports.CrossSigningInfo = CrossSigningInfo;

function deviceToObject(device, userId) {
  return {
    algorithms: device.algorithms,
    keys: device.keys,
    device_id: device.deviceId,
    user_id: userId,
    signatures: device.signatures
  };
}

var CrossSigningLevel = {
  MASTER: 4,
  USER_SIGNING: 2,
  SELF_SIGNING: 1
};
/**
 * Represents the ways in which we trust a user
 */

exports.CrossSigningLevel = CrossSigningLevel;

var UserTrustLevel = /*#__PURE__*/function () {
  function UserTrustLevel(crossSigningVerified, crossSigningVerifiedBefore, tofu) {
    (0, _classCallCheck2["default"])(this, UserTrustLevel);
    this._crossSigningVerified = crossSigningVerified;
    this._crossSigningVerifiedBefore = crossSigningVerifiedBefore;
    this._tofu = tofu;
  }
  /**
   * @returns {bool} true if this user is verified via any means
   */


  (0, _createClass2["default"])(UserTrustLevel, [{
    key: "isVerified",
    value: function isVerified() {
      return this.isCrossSigningVerified();
    }
    /**
     * @returns {bool} true if this user is verified via cross signing
     */

  }, {
    key: "isCrossSigningVerified",
    value: function isCrossSigningVerified() {
      return this._crossSigningVerified;
    }
    /**
     * @returns {bool} true if we ever verified this user before (at least for
     * the history of verifications observed by this device).
     */

  }, {
    key: "wasCrossSigningVerified",
    value: function wasCrossSigningVerified() {
      return this._crossSigningVerifiedBefore;
    }
    /**
     * @returns {bool} true if this user's key is trusted on first use
     */

  }, {
    key: "isTofu",
    value: function isTofu() {
      return this._tofu;
    }
  }]);
  return UserTrustLevel;
}();
/**
 * Represents the ways in which we trust a device
 */


exports.UserTrustLevel = UserTrustLevel;

var DeviceTrustLevel = /*#__PURE__*/function () {
  function DeviceTrustLevel(crossSigningVerified, tofu, localVerified, trustCrossSignedDevices) {
    (0, _classCallCheck2["default"])(this, DeviceTrustLevel);
    this._crossSigningVerified = crossSigningVerified;
    this._tofu = tofu;
    this._localVerified = localVerified;
    this._trustCrossSignedDevices = trustCrossSignedDevices;
  }

  (0, _createClass2["default"])(DeviceTrustLevel, [{
    key: "isVerified",

    /**
     * @returns {bool} true if this device is verified via any means
     */
    value: function isVerified() {
      return Boolean(this.isLocallyVerified() || this._trustCrossSignedDevices && this.isCrossSigningVerified());
    }
    /**
     * @returns {bool} true if this device is verified via cross signing
     */

  }, {
    key: "isCrossSigningVerified",
    value: function isCrossSigningVerified() {
      return this._crossSigningVerified;
    }
    /**
     * @returns {bool} true if this device is verified locally
     */

  }, {
    key: "isLocallyVerified",
    value: function isLocallyVerified() {
      return this._localVerified;
    }
    /**
     * @returns {bool} true if this device is trusted from a user's key
     * that is trusted on first use
     */

  }, {
    key: "isTofu",
    value: function isTofu() {
      return this._tofu;
    }
  }], [{
    key: "fromUserTrustLevel",
    value: function fromUserTrustLevel(userTrustLevel, localVerified, trustCrossSignedDevices) {
      return new DeviceTrustLevel(userTrustLevel._crossSigningVerified, userTrustLevel._tofu, localVerified, trustCrossSignedDevices);
    }
  }]);
  return DeviceTrustLevel;
}();

exports.DeviceTrustLevel = DeviceTrustLevel;

function createCryptoStoreCacheCallbacks(store) {
  return {
    getCrossSigningKeyCache: function getCrossSigningKeyCache(type, _expectedPublicKey) {
      return new Promise(function (resolve) {
        return store.doTxn('readonly', [_indexeddbCryptoStore.IndexedDBCryptoStore.STORE_ACCOUNT], function (txn) {
          store.getSecretStorePrivateKey(txn, resolve, type);
        });
      });
    },
    storeCrossSigningKeyCache: function storeCrossSigningKeyCache(type, key) {
      if (!(key instanceof Uint8Array)) {
        throw new Error("storeCrossSigningKeyCache expects Uint8Array, got ".concat(key));
      }

      return store.doTxn('readwrite', [_indexeddbCryptoStore.IndexedDBCryptoStore.STORE_ACCOUNT], function (txn) {
        store.storeSecretStorePrivateKey(txn, type, key);
      });
    }
  };
}